@@ -407,49 +407,10 @@ R_xlen_t do_which_first_xd_yd(const double * x,
                               int op,
                               const double * y,
                               R_xlen_t N) {
-  switch(op) {
-  case OP_NE:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] != y[i]) {
-        return i + 1;
-      }
+  for (R_xlen_t i = 0; i < N; ++i) {
+    if (dsingle_ox_x1_x2(x[i], op, y[i], 0)) {
+      return i + 1;
     }
-    break;
-  case OP_EQ:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] == y[i]) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_GE:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] >= y[i]) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_GT:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] > y[i]) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_LE:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] <= y[i]) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_LT:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] < y[i]) {
-        return i + 1;
-      }
-    }
-    break;
   }
 
   return 0;
@@ -614,28 +575,13 @@ R_xlen_t do_which_first_xi_aii(const int * x, int op, int a1, int a2, R_xlen_t N
   if (a2 == NA_INTEGER) {
     a2 = INT_MAX;
   }
-  switch(op) {
-  case OP_BW:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] >= a1 && x[i] <= a2) {
-        return i + 1;
-      }
+  if (a1 > a2) {
+    return 0;
+  }
+  for (R_xlen_t i = 0; i < N; ++i) {
+    if (isingle_ox_x1_x2(x[i], op, a1, a2)) {
+      return i + 1;
     }
-    break;
-  case OP_BO:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] > a1 && x[i] < a2) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_BC:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] <= a1 || x[i] >= a2) {
-        return i + 1;
-      }
-    }
-    break;
   }
   return 0;
 }
@@ -645,34 +591,20 @@ R_xlen_t do_which_first_xd_add(const double * x,
                                double a1,
                                double a2,
                                R_xlen_t N) {
+
   if (ISNAN(a1)) {
     a1 = R_NegInf;
   }
   if (ISNAN(a2)) {
     a2 = R_PosInf;
   }
-  switch(op) {
-  case OP_BW:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] >= a1 && x[i] <= a2) {
-        return i + 1;
-      }
+  if (a1 > a2) {
+    return 0;
+  }
+  for (R_xlen_t i = 0; i < N; ++i) {
+    if (dsingle_ox_x1_x2(x[i], op, a1, a2)) {
+      return i + 1;
     }
-    break;
-  case OP_BO:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] > a1 && x[i] < a2) {
-        return i + 1;
-      }
-    }
-    break;
-  case OP_BC:
-    for (R_xlen_t i = 0; i < N; ++i) {
-      if (x[i] <= a1 || x[i] >= a2) {
-        return i + 1;
-      }
-    }
-    break;
   }
   return 0;
 }
@@ -850,13 +782,16 @@ R_xlen_t do_which_first_xi_ad(const int * x,
 }
 
 R_xlen_t do_which_first_xi_add(const int * x, int op, double a1, double a2, R_xlen_t N) {
+
   if (ISNAN(a1) || a1 < -2147483647) {
     a1 = R_NegInf;
   }
   if (ISNAN(a2) || a1 > 2147483647) {
     a2 = R_PosInf;
   }
-
+  if (a1 > a2) {
+    return 0;
+  }
   switch(op) {
   case OP_BW:
     for (R_xlen_t i = 0; i < N; ++i) {
